@@ -35,7 +35,7 @@ protected:
     {
         DataType *old = data;
         data = new DataType[newSize];
-        for (SizeType i=0;i<size;++i) data[i] = old[i];
+        for (SizeType i = 0;i<size;++i) data[i] = old[i];
         delete[] old;
         capa = newSize;
     }
@@ -45,25 +45,34 @@ protected:
     // shall not be copyable
     YVec(const YVec& other);
     YVec operator=(const YVec& other);
+    YVec& operator=(YVec&& other) { other.swap(*this); return *this; }
 
 public:
     SizeType size;
     DataType *data;
-    inline YVec(): capa(0), size(0), data(0) {}
+    inline YVec(): capa(0), size(0), data(nullptr) {}
     inline YVec(SizeType initialCapa):  capa(initialCapa), size(0), data(new DataType[initialCapa]) { }
+    inline YVec(YVec&& src) { src.swap(*this); }
+
     inline void reset() {
         if (!size) return;
-        delete[] data; data = 0;
+        delete[] data; data = nullptr;
         capa = size = 0;
     }
     inline void preserve(SizeType wanted) { if (wanted > capa) resize(wanted); }
     inline SizeType remainingCapa() { return capa - size; }
     inline ~YVec() { reset(); }
+    void swap(YVec& other) { if(this == &other) return; ::swap(data, other.data); ::swap(size, other.size); }
     DataType& add(const DataType& element) {
         if (size >= capa)
             inflate();
         return data[size++] = element;
     }
+    DataType& emplace_back(DataType&& element) {
+        if (size >= capa)
+            inflate();
+        return data[size++] = static_cast<DataType&&>(element);
+    }
     DataType& insert(const DataType& element, SizeType destPos)
     {
         if (size >= capa)
@@ -71,9 +80,9 @@ public:
         if (size > destPos) {
             DataType *old = data;
             data = new DataType[size + 1];
-            for (SizeType i=0; i < destPos; ++i)
+            for (SizeType i = 0; i < destPos; ++i)
                 data[i] = old[i];
-            for (SizeType i=destPos; i < size; ++i)
+            for (SizeType i = destPos; i < size; ++i)
                 data[i+1] = old[i];
             delete[] old;
             capa = size + 1;
@@ -89,6 +98,9 @@ public:
     inline iterator getIterator(bool reverse = false) {
         return iterator(this, reverse);
     }
+    // STL-friendly iterator
+    DataType* begin() { return data; }
+    DataType* end() { return data + size; }
 };
 /**
  * Simple container based on YVec but made only for raw pointers of the particular type.
@@ -184,17 +196,17 @@ public:
         YKeyValuePair<kvp*,kvp*> ret(NULL, NULL);
         if (!binsearch(key, pos))
             return ret;
-        for(int i=pos;;++i)
+        for (int i = pos;;++i)
         {
-            if(i==store.size || !values_equal(i, pos))
+            if (i == store.size || !values_equal(i, pos))
             {
                 ret.value = &store[i];
                 break;
             }
         }
-        for(int i=pos;;--i)
+        for (int i = pos;;--i)
         {
-            if(i==0 || !values_equal(i-1, pos))
+            if (i == 0 || !values_equal(i-1, pos))
             {
                 ret.key = &store[i];
                 break;
@@ -213,7 +225,7 @@ public:
             return;
         }
         store.preserve(store.size+1);
-        int destPos=0;
+        int destPos = 0;
 
         binsearch(key, destPos);
         // cursor ended on the left side?
