@@ -70,6 +70,7 @@ YMenu::YMenu(YWindow *parent):
     fTimerX = 0;
     fTimerY = 0;
     fTimerSubmenuItem = -1;
+    fKeyPressed = 0;
     addStyle(wsNoExpose);
 
     if (menuFont == null)
@@ -318,10 +319,9 @@ bool YMenu::handleKey(const XKeyEvent &key) {
     int m = KEY_MODMASK(key.state);
 
     if (key.type == KeyPress) {
+        fKeyPressed = k;
         if ((m & ~ShiftMask) == 0) {
-            if (k == XK_Escape) {
-                cancelPopup();
-            } else if (k == XK_Left || k == XK_KP_Left) {
+            if (k == XK_Left || k == XK_KP_Left) {
                 if (prevPopup())
                     cancelPopup();
             } else if (itemCount() > 0) {
@@ -377,6 +377,12 @@ bool YMenu::handleKey(const XKeyEvent &key) {
             }
         }
     }
+    else if (key.type == KeyRelease) {
+        if (k == XK_Escape && k == fKeyPressed && m == 0) {
+            YPopupWindow::finishPopup();
+            return true;
+        }
+    }
     return YPopupWindow::handleKey(key);
 }
 
@@ -657,6 +663,13 @@ YMenuItem *YMenu::addSubmenu(const mstring &name, int hotCharPos, YMenu *submenu
     return add(new YMenuItem(name, hotCharPos, null, actionNull, submenu));
 }
 
+YMenuItem *YMenu::addSubmenu(const mstring &name, int hotCharPos,
+                             YMenu *submenu, YMenuItem* after)
+{
+    YMenuItem* item = new YMenuItem(name, hotCharPos, null, actionNull, submenu);
+    return add(item, after);
+}
+
 void YMenu::addSeparator() {
     int n = itemCount();
     if (n > 0 && !fItems[n - 1]->isSeparator()) {
@@ -678,6 +691,21 @@ YMenuItem* YMenu::lastItem() const {
     return n > 0 ? fItems[n - 1] : nullptr;
 }
 
+bool YMenu::removeSubmenu(YMenu* menu) {
+    bool done = false;
+    YMenuItem* item = findSubmenu(menu);
+    if (item) {
+        hideSubmenu();
+        int index = find(fItems, item);
+        if (index >= 0) {
+            fItems[index]->setSubmenu(nullptr);
+            fItems.remove(index);
+        }
+        done = true;
+    }
+    return done;
+}
+
 void YMenu::removeAll() {
     hideSubmenu();
     for (YMenuItem* item : fItems) {
@@ -693,6 +721,17 @@ YMenuItem * YMenu::add(YMenuItem *item) {
     return item;
 }
 
+YMenuItem * YMenu::add(YMenuItem* item, YMenuItem* after) {
+    if (item) {
+        int i = find(fItems, after);
+        if (i >= 0)
+            fItems.insert(i + 1, item);
+        else
+            fItems.append(item);
+    }
+    return item;
+}
+
 YMenuItem * YMenu::add(YMenuItem *item, const char *icons) {
     if (item) {
         ref<YIcon> icon(YIcon::getIcon(icons));
@@ -727,6 +766,10 @@ YMenuItem * YMenu::addSorted(YMenuItem *item, bool duplicates, bool ignoreCase)
     return item;
 }
 
+bool YMenu::haveCommand(YAction action) {
+    return findAction(action) != nullptr;
+}
+
 YMenuItem *YMenu::findAction(YAction action) {
     for (YMenuItem* item : fItems)
         if (action == item->getAction())
@@ -784,6 +827,13 @@ void YMenu::checkCommand(YAction action, bool check) {
             item->setChecked(check);
 }
 
+void YMenu::removeCommand(YAction action) {
+    int i = -1;
+    for (YMenuItem* item : fItems)
+        if (++i, action == item->getAction())
+            return fItems.remove(i);
+}
+
 void YMenu::getOffsets(int &left, int &top, int &right, int &bottom) {
     if (wmLook == lookMetal || wmLook == lookFlat) {
         left = 1;
@@ -916,6 +966,12 @@ void YMenu::sizePopup(int hspace) {
     setSize(unsigned(width), unsigned(height));
 }
 
+void YMenu::repaintItem(YMenuItem* item) {
+    int id = find(fItems, item);
+    if (0 <= id)
+        repaintItem(id);
+}
+
 void YMenu::repaintItem(int item) {
     int x, y;
     unsigned h;
